from alliancepy.team import Team

# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Client:
    """
    This is the main client class used for accessing the TOA API. Currently, it only serves up :class:`~.team.Team`
    objects but it will
    expand over time.

    :param api_key: Your TOA API key. This is required, otherwise you will not be able to access the database.
    :type api_key: str
    :param application_name: The name of the application that you are using to access the API. \
    This can just be the name of your script.
    :type application_name: str

    """

    def __init__(self, api_key: str, application_name: str):
        self._headers = {
            "content-type": "application/json",
            "x-toa-key": api_key,
            "x-application-origin": application_name,
        }

    def team(self, team_number: int):
        """Create a :class:`~.team.Team` object.

        :param team_number: A valid First Tech Challenge team number.
        :type team_number: int
        :return: The Team object
        :rtype: :class:`~.team.Team`
        """
        return Team(team_number=team_number, headers=self._headers)



from alliancepy.http import request


# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Match:
    """
    An object containing the details of an FTC match. Instances of this class should not be created directly. Instead,
    use your :class:`~alliancepy.event.Event` object.

    randomization
        The randomization of the stones and skystones at the beginning of the match.
    red
        An :class:`Alliance` object containing the details of the red alliance of the match.
    blue
        An :class:`Alliance` object containing the details of the blue alliance of the match.
    """

    def __init__(self, match_key: str, headers: dict):
        self._match_key = match_key
        self._headers = headers
        details = request(f"/match/{self._match_key}/details", headers=self._headers)
        self.randomization = int(details[0]["randomization"])
        self.red = Alliance("red", self._match_key, details, self._headers)
        self.blue = Alliance("blue", self._match_key, details, self._headers)

    def __str__(self):
        return f"<Match ({self._match_key})>"

    def __repr__(self):
        return str(self)

    @property
    def participants(self):
        """
        The participants of the match.

        :return: The team numbers in a list
        :rtype: List[int]
        """
        participants = request(
            f"/match/{self._match_key}/participants", headers=self._headers
        )
        x = []
        for part in participants:
            raw = part["team_key"]
            x.append(int(raw))
        return x


class Alliance:
    """
    An object representing a match alliance. Instances of this class should not be created directly. Instead,
    use your :class:`Match` object.

    robot_1
        A :class:`Robot` object that represents the first team's robot.
    robot_2
        A :class:`Robot` object that represents the second team's robot.
    """

    def __init__(self, alliance: str, match_key: str, details: list, headers: dict):
        self._alliance = alliance
        self._details = details[0]
        self._headers = headers
        self.robot_1 = Robot(self._alliance, 1, match_key, details, self._headers)
        self.robot_2 = Robot(self._alliance, 2, match_key, details, self._headers)

    def __str__(self):
        return f"<Alliance ({self._alliance})>"

    def __repr__(self):
        return str(self)

    @property
    def min_penalty(self):
        """
        The amount of minor penalties an alliance earned.

        :rtype: int
        """
        key = f"{self._alliance}_min_pen"
        return int(self._details[key])

    @property
    def maj_penalty(self):
        """
        The amount of major penalites an alliance earned.

        :rtype: int
        """
        key = f"{self._alliance}_maj_pen"
        return int(self._details[key])

    @property
    def auto_stones(self):
        """A list of the stones the alliance stacked in autonomous.

        :rtype: List[str]
        """
        x = []
        for item in self._details[self._alliance]:
            if "auto_stone_" in item:
                x.append(self._details[self._alliance][item])
        return x

    @property
    def foundation(self):
        """
        Whether the alliance repositioned the foundation suring autonomous or not.

        :rtype: bool
        """
        return self._details[self._alliance]["foundation_repositioned"]

    @property
    def teleop(self):
        """
        Returns a dict containing integer values.
        Contains "delivered", "placed", and "returned".

        :rtype: dict[str: int]
        """
        delivered = self._details[self._alliance]["tele_delivered"]
        placed = self._details[self._alliance]["tele_placed"]
        returned = self._details[self._alliance]["tele_returned"]
        x = {"delivered": delivered, "returned": returned, "placed": placed}
        return x


class Robot:
    """
    An object containing details about a robot. Instances of this class should not be created directly. Instead,
    use your :class:`Alliance` object.
    """

    def __init__(
            self,
            alliance: str,
            robot_number: int,
            match_key: str,
            details: list,
            headers: dict,
    ):
        self._alliance = alliance
        self._robot_number = robot_number
        self._match_key = match_key
        self._details = details[0]
        self._headers = headers

    @property
    def parked_skybridge(self):
        """Whether the robot parked under the skybridge or not

        :rtype: bool
        """
        key = f"robot_{self._robot_number}"
        value = self._details[self._alliance][key]["nav"]
        return bool(value)

    @property
    def parked_endgame(self):
        """
        Whether the robot parked at the end of the match or not.

        :rtype: bool
        """
        key = f"robot_{self._robot_number}"
        value = self._details[self._alliance][key]["parked"]
        return bool(value)

    @property
    def capstone_level(self):
        """
        The level of the capstone at the end of the match.

        :rtype: int
        """
        key = f"robot_{self._robot_number}"
        value = self._details[self._alliance][key]["parked"]
        return int(value)

    @property
    def owner(self):
        """The team that owns the bot.

        :return: The team's team number as an integer
        :rtype: int
        """
        match = request(f"/match/{self._match_key}", headers=self._headers)
        participants = list(filter(lambda p: p["station_status"] == 1, match[0]["participants"]))
        if self._alliance == "red" and self._robot_number == 1:
            raw = participants[0]["team_key"]
        elif self._alliance == "red" and self._robot_number == 2:
            raw = participants[1]["team_key"]
        elif self._alliance == "blue" and self._robot_number == 1:
            raw = participants[2]["team_key"]
        elif self._alliance == "blue" and self._robot_number == 2:
            raw = participants[3]["team_key"]
        else:
            raise ValueError("Something went wrong, please try again")
        return int(raw)



from .async_http import request
from .async_event import Event
from alliancepy.season import Season
import asyncio
from concurrent.futures import ThreadPoolExecutor
import re

# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Team:
    """
    This is the Asynchronous version of the normal :class:`~alliancepy.team.Team` class. You should not create your own
    instances of this class - instead use your :class:`~.async_client.AsyncClient` object.

    region
        The key of the team's according region.
    league
        The key of the team's league.
    short_name
        The short team name.
    long_name
        The long team name.
    robot_name
        The name of the team's robot.
    location
        The team's location, in "City, State/Province, Country, Zipcode" format.
    rookie_year
        The year in which the team was a rookie team.
    last_active
        The season key of the season in which the team participated in their most recent match.
    website
        The URL of the team's website, if they have any

    """

    def __init__(self, team_number, headers: dict):
        self._team_number = team_number
        self._headers = headers
        self._loop = asyncio.get_event_loop()
        team = self._loop.run_until_complete(
            request(target=f"/team/{team_number}", headers=headers)
        )
        team = team[0]
        self.region = team["region_key"]
        self.league = team["league_key"]
        self.short_name = team["team_name_short"]
        self.long_name = team["team_name_long"]
        self.robot_name = team["robot_name"]
        location = f"{team['city']}, {team['state_prov']}, {team['country']}, {team['zip_code']}"
        self.location = location
        self.rookie_year = team["rookie_year"]
        self.last_active = team["last_active"]
        self.website = team["website"]

    async def events(self, season: Season):
        """
        Every event the team has participated in, in a particular season.

        :param season: An alliancepy Season object
        :type season: :class:`~.season.Season`
        :return: A dict containing the :class:`~.event.Event` objects. The key names are shortened versions of the TOA
        event key.
        :rtype: dict
        """
        edict = {}
        events = await request(
            f"/team/{self._team_number}/events/{season}", headers=self._headers
        )

        def _parse_events(ev=events, ed=None):
            ed = ed or edict
            for event in ev:
                e = Event(event_key=event["event_key"], headers=self._headers)
                event_key = event["event_key"]
                raw_key = str(e.name)
                key = raw_key.replace(" ", "_")
                key = key.lower()
                if key in ed:
                    raw_key_right = re.sub(r"\d{4}-\w+-", "", event_key)
                    raw_key_right = raw_key_right.lower()
                    key = f"{key}_{raw_key_right}"
                ed[key] = e

            return ed

        loop = asyncio.get_event_loop_policy().new_event_loop()
        future = loop.run_in_executor(ThreadPoolExecutor(), _parse_events)
        return loop.run_until_complete(future)

    async def _wlt(self):
        data = await request(
            target=f"/team/{self._team_number}/wlt", headers=self._headers
        )
        self._wins = data[0]["wins"]
        self._losses = data[0]["losses"]
        self._ties = data[0]["ties"]

    @property
    def wins(self):
        """
        The total amount of times the team has won a match.

        :return: The number of wins.
        :rtype: int
        """
        self._loop.run_until_complete(self._wlt())
        return self._wins

    @property
    def losses(self):
        """
        The total amount of times the team has lost a match.

        :return: The number of wins.
        :rtype: int
        """
        self._loop.run_until_complete(self._wlt())
        return self._losses

    @property
    def ties(self):
        """
        The total amount of times the team has tied in a match.

        :return: The number of wins.
        :rtype: int
        """
        self._loop.run_until_complete(self._wlt())
        return self._ties

    async def _rankings(self, season: Season):
        rankings = await request(
            f"/team/{self._team_number}/results/{season}", headers=self._headers
        )
        return rankings

    async def season_wins(self, season: Season):
        """
        The amount of times a team has won a match in a particular season.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["wins"]
            x.append(int(raw))

        return sum(x)

    async def season_losses(self, season: Season):
        """
        The amount of times a team has lost a match in a particular season.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["losses"]
            x.append(int(raw))

        return sum(x)

    async def season_ties(self, season: Season):
        """
        The amount of times a team has tied in a match in a particular season.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["ties"]
            x.append(int(raw))

        return sum(x)

    async def opr(self, season: Season):
        """
        OPR stands for Offensive Power Rating, which is a system to attempt to deduce the average point contribution of
        a team to an alliance. Penalties are also factored in.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["opr"]
            x.append(int(raw))

        return sum(x)

    async def np_opr(self, season: Season):
        """
        NP_OPR is just OPR, but penalties are not factored in.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["np_opr"]
            x.append(int(raw))

        return sum(x)

    async def tiebreaker_points(self, season: Season):
        """
        Tiebreaker points are the pre-penalty score of the losing alliance for each match. This function returns the
        total tiebreaker points of a team in one season.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["tie_breaker_points"]
            x.append(int(raw))

        return sum(x)

    async def ranking_points(self, season: Season):
        """
        Ranking points are the number of points scored by the losing alliance in a qualification match.
        If you win the match, then the RP awarded to you is the score of your opponent alliance (which lost).
        If you lose the match, then the RP awarded to you is your own alliance’s score.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["ranking_points"]
            x.append(int(raw))

        return sum(x)

    async def qualifying_points(self, season: Season):
        """
        Winning teams of a qualifying match eatch receive 2 QP. Losing teams receive 0. If a match ends in a tie, all
        four teams receive 1 QP.

        :param season: A valid TOA season key
        :type season: :class:`~alliancepy.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = await self._rankings(season)
        x = []
        for item in data:
            raw = item["qualifying_points"]
            x.append(int(raw))

        return sum(x)



import asyncio
import sys

# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

if sys.platform == "win32" and hasattr(asyncio, "WindowsSelectorEventLoopPolicy"):
    _BasePolicy = asyncio.WindowsSelectorEventLoopPolicy
else:
    _BasePolicy = asyncio.DefaultEventLoopPolicy


class ThreadEventLoopPolicy(_BasePolicy):
    def get_event_loop(self):
        try:
            return super().get_event_loop()
        except (RuntimeError, AssertionError):
            loop = self.new_event_loop()
            self.set_event_loop(loop)
            return loop



from .async_http import request
from .async_match import Match
from alliancepy.season import Season
from alliancepy.match_type import MatchType
import asyncio
import re

# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Event:
    """
    This is the asynchronous version of the normal :class:`~alliancepy.event.Event` class.
    Instances of this class should not be created directly; instead use your :class:`~.team.Team` object.

    season
        The season in which the event occurred.
    region
        The key of the region in which the event occured.
    league
        The key of the league the event occured in, if any.
    name
        The name of the event
    location
        The location of the event, in City, State/Province, Country form.
    venue
        The venue of the event.
    """

    def __init__(self, event_key: str, headers: dict):
        self._event_key = event_key
        self._headers = headers
        self._loop = asyncio.get_event_loop()
        info = self._loop.run_until_complete(
            request(f"/event/{self._event_key}", headers=self._headers)
        )
        info = info[0]
        season_key = int(info["season_key"])
        self.season = Season(season_key).name
        self.region = info["region_key"]
        self.league = info["league_key"]
        self.name = info["event_name"]
        location = f"{info['city']} {info['state_prov']}, {info['country']}"
        self.location = location
        self.venue = info["venue"]

    def __str__(self):
        return f"<Event: {self.name}>"

    def __repr__(self):
        return f"<Event: {self.name} ({self._event_key})>"

    async def match(self, match_type: MatchType, match_number: int):
        """
        Get one of the matches for the event.

        :param match_type: The type of the match. See :ref:`match_type` for more information.
        :type match_type: :class:`~alliancepy.match_type.MatchType`
        :param match_number: The number of the match.
        :type match_number: int
        :return: A :class:`~.async_match.Match` object containing details about the specific match.
        :rtype: :class:`.async_match.Match`
        """
        if len(str(match_number)) == 1:
            match_name = f"{match_type.value}00{match_number}"
        elif len(str(match_number)) == 2:
            match_name = f"{match_type.value}0{match_number}"
        else:
            match_name = f"{match_type.value}{match_number}"
        loop = asyncio.get_event_loop()
        matches = loop.run_until_complete(
            request(f"/event/{self._event_key}/matches", headers=self._headers)
        )
        mdict = {}
        for match in matches:
            key = match["match_key"]
            key_right_strip = re.sub(r"\d{4}-\w+-\w+-", "", key)
            value = re.sub(r"-\d+", "", key_right_strip)
            mdict[key] = value
        try:
            match_key = list(mdict.keys())[
                list(mdict.values()).index(match_name.upper())
            ]
        except ValueError:
            raise ValueError("This match does not exist")
        else:
            return Match(match_key, headers=self._headers)

    async def _rankings(self):
        resp = await request(
            f"/event/{self._event_key}/rankings", headers=self._headers
        )
        return resp

    async def rank(self, team_number: int):
        """
        The specified team's rank at the end of the match.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The rank as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["rank"]
                return int(raw)

    async def rank_change(self, team_number: int):
        """
        The amount of times the team's rank changed during the event.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The rank change as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["rank_change"]
                return int(raw)

    async def wins(self, team_number: int):
        """
        The amount of times within the event that the specified team won a match.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The amount of wins as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["wins"]
                return int(raw)

    async def losses(self, team_number: int):
        """
        The amount of times within the event that the specified team lost a match.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The amount of losses as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["losses"]
                return int(raw)

    async def ties(self, team_number: int):
        """
        The amount of times within the event that the specified team tied in a match.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The amount of ties as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["ties"]
                return int(raw)

    async def opr(self, team_number: int):
        """
        The specified team's OPR for this event only. Penalties are factored in.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The OPR as a floating point number.
        :rtype: float
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["opr"]
                return int(raw)

    async def np_opr(self, team_number: int):
        """
        The specified team's OPR for this event only. Penaltied are not factored in.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The NP_OPR as a floating point number.
        :rtype: float
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["np_opr"]
                return int(raw)

    async def highest_qualifier_score(self, team_number: int):
        """
        The specified team's highest score in a qualifier.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The score as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["highest_qual_score"]
                return int(raw)

    async def ranking_points(self, team_number: int):
        """
        The specified team's ranking points for this event only.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The amount of ranking points as a floating point number
        :rtype: float
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["ranking_points"]
                return int(raw)

    async def qualifying_points(self, team_number: int):
        """
        The specified team's qualifying points for this event only.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The amount of qualifying points as an integer
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["qualifying_points"]
                return int(raw)

    async def tiebreaker_points(self, team_number: int):
        """
        The specified team's tiebreaker points for this event only.

        :param team_number: A valid FTC team number.
        :type team_number: int
        :return: The amount of tiebreaker points as a floating point number
        :rtype: int
        """
        rankings = await self._rankings()
        for rank in rankings:
            if rank["team"]["team_number"] == team_number:
                raw = rank["tie_breaker_points"]
                return int(raw)



import re
from alliancepy.http import request
from alliancepy.season import Season
from alliancepy.event import Event

# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Team:
    """
    This is the class used to access an existing FTC team. Do not create instances of this class yourself. Instead use
    the "team" method provided by your client object.

    region
        The key of the team's according region.
    league
        The key of the team's league.
    short_name
        The short team name.
    long_name
        The long team name.
    robot_name
        The name of the team's robot.
    location
        The team's location, in "City, State/Province, Country, Zipcode" format.
    rookie_year
        The year in which the team was a rookie team.
    last_active
        The season key of the season in which the team participated in their most recent match.
    website
        The URL of the team's website, if they have any

    """

    def __init__(self, team_number: int, headers: dict):
        self._team_number = team_number
        self._headers = headers
        team = request(target=f"/team/{team_number}", headers=headers)
        team = team[0]
        self.region = team["region_key"]
        self.league = team["league_key"]
        self.short_name = team["team_name_short"]
        self.long_name = team["team_name_long"]
        self.robot_name = team["robot_name"]
        location = f"{team['city']}, {team['state_prov']}, {team['country']}, {team['zip_code']}"
        self.location = location
        self.rookie_year = team["rookie_year"]
        self.last_active = team["last_active"]
        self.website = team["website"]

    def events(self, season: Season):
        """
        Every event the team has participated in, in a particular season.

        :param season: An alliancepy Season object
        :type season: :class:`~.season.Season`
        :return: A dict containing the :class:`~.event.Event` objects. The key names are shortened versions of the TOA
        event key.
        :rtype: dict
        """
        edict = {}
        events = request(
            f"/team/{self._team_number}/events/{season}", headers=self._headers
        )
        for event in events:
            event_key = event["event_key"]
            e = Event(event_key=event_key, headers=self._headers)
            raw_key = str(e.name)
            key = raw_key.replace(" ", "_")
            key = key.lower()
            if key in edict:
                raw_key_right = re.sub(r"\d{4}-\w+-", "", event_key)
                raw_key_right = raw_key_right.lower()
                key = f"{key}_{raw_key_right}"
            edict[key] = e

        return edict

    def _wlt(self):
        data = request(target=f"/team/{self._team_number}/wlt", headers=self._headers)
        return data[0]

    @property
    def wins(self):
        """
        The total amount of times the team has won a match.

        :return: The number of wins.
        :rtype: int
        """
        data = self._wlt()["wins"]
        return int(data)

    @property
    def losses(self):
        """
        The total amount of times the team has lost a match.
        :return: The number of losses.
        :rtype: int
        """
        data = self._wlt()["losses"]
        return int(data)

    @property
    def ties(self):
        """
        The total amount of times the team has tied in a match.

        :return: The number of ties.
        :rtype: int
        """
        data = self._wlt()["ties"]
        return int(data)

    def _rankings(self, season: Season):
        rankings = request(
            f"/team/{self._team_number}/results/{season}", headers=self._headers
        )
        return rankings

    def season_wins(self, season: Season):
        """
        The amount of times a team has won a match in a particular season.

        :param season: A valid TOA season key
        :type season: :class:`~.season.Season`
        :return: The number of wins in the specified season
        :rtype: int
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["wins"]
            x.append(int(raw))
        return sum(x)

    def season_losses(self, season: Season):
        """
        The amount of times a team has lost a match in a particular season.

        :param season: A valid TOA season key
        :type season: :class:`~.season.Season`
        :return: The number of losses in the specified season
        :rtype: int
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["losses"]
            x.append(int(raw))
        return sum(x)

    def season_ties(self, season: Season):
        """
        The amount of times a team has tied in a match in a particular season.

        :param season: A valid TOA season key
        :type season: :class:`~.season.Season`
        :return: The number of ties in the specified season
        :rtype: int
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["ties"]
            x.append(int(raw))
        return sum(x)

    def opr(self, season: Season):
        """
        OPR stands for Offensive Power Rating, which is a system to attempt to deduce the average point contribution of
        a team to an alliance. Penalties are also factored in.

        :param season: A valid TOA season key.
        :type season: :class:`~.season.Season`
        :return: The team's OPR in the specified season
        :rtype: float
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["opr"]
            x.append(float(raw))
        return sum(x)

    def np_opr(self, season: Season):
        """
        NP_OPR is just OPR, but penalties are not factored in.

        :param season: A valid TOA season key.
        :type season: :class:`~.season.Season`
        :return: The team's NP_OPR (OPR without Penalties) in the specified season
        :rtype: float
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["np_opr"]
            x.append(float(raw))
        return sum(x)

    def tiebreaker_points(self, season: Season):
        """Tiebreaker points are the pre-penalty score of the losing alliance for each match. This function returns the
        total tiebreaker points of a team in one season.

        :param season: A valid TOA season key.
        :type season: :class:`~.season.Season`
        :return: The team's tiebreaker points in the specified season
        :rtype: float
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["tie_breaker_points"]
            x.append(float(raw))
        return sum(x)

    def ranking_points(self, season: Season):
        """Ranking points are the number of points scored by the losing alliance in a qualification match.
        If you win the match, then the RP awarded to you is the score of your opponent alliance (which lost).
        If you lose the match, then the RP awarded to you is your own alliance's score.

        :param season: A valid TOA season key.
        :type season: :class:`~.season.Season`
        :return: The team's ranking points in the specified season
        :rtype: float
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["ranking_points"]
            x.append(float(raw))
        return sum(x)

    def qualifying_points(self, season: Season):
        """
        Winning teams of a qualifying match eatch receive 2 QP. Losing teams receive 0. If a match ends in a tie, all
        four teams receive 1 QP.


        :param season: A valid TOA season key.
        :type season: :class:`~.season.Season`
        :return: The team's qualifying points in the specified season
        :rtype: int
        """
        data = self._rankings(season)
        x = []
        for item in data:
            raw = item["qualifying_points"]
            x.append(int(raw))

        return sum(x)



from setuptools import setup
import re

# MIT License
#
# Copyright (c) 2020 Yash Karandikar
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

with open("README.rst") as f:
    readme = f.read()

with open("alliancepy/__init__.py") as f:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE
    ).group(1)

extras_require = {
    "docs": ["sphinx", "sphinx-rtd-theme"],
    "async": ["aiohttp[speedups]", "nest_asyncio"],
}

setup(
    name="alliancepy",
    version=version,
    packages=["alliancepy", "alliancepy.ext.aio"],
    url="https://github.com/karx1/alliancepy",
    license="MIT",
    author="karx",
    author_email="nerdstep710@gmail.com",
    description="A library to access The Orange Alliance API",
    long_description=readme,
    python_requires=">=3.6",
    extras_require=extras_require,
    test_suite="nose.collector",
    tests_require=["nose"],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
    ],
)
